/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Conf facility.
 *
 * This file handles the global configuration of the daemon.
 *
 * See conf.h for more information on the functions 
 *
 */

/* NOTE:
 *
 * In case a "reload configuration" function has to be written, this file may need to be made re-entrant.
 * It will require work on every place in the code of the daemon that uses "g_conf" and "g_pconf".
 */

#include "waaad-internal.h"
#include <string.h>
#include <errno.h>
#include <stdio.h>
 
waaad_conf_t * g_conf=NULL;

static waaad_conf_t waaad_config;

/* Set the default values in a configuration structure */
static int conf_setdefaults(waaad_conf_t * conf)
{
	/* Clear the configuration */
	memset(conf, 0, sizeof(waaad_conf_t));
	
	/* log to console */
	conf->log_to = LOG_TO_CONSOLE;
	
	/* The default configuration file path (from configure) */
	conf->filepath = DEFAULT_CONF_FILE;
	
	/* The default tctimer value */
	conf->pub.tctimer = 30;
	
	return 0;
}

/* Output the parsed configuration */
static void conf_dump(void)
{
	TRACE_DEBUG( INFO, "--- Parsed configuration : ---");
	TRACE_DEBUG( INFO, "  Diameter-Id : %s", g_pconf->diameter_identity);
	TRACE_DEBUG( INFO, "  Tc timer    : %d", g_pconf->tctimer);
	TRACE_DEBUG( INFO, "------------------------------");
}

/* Initialize the configuration module */
int conf_init ( void )
{
	/* Set the global pointer */
	g_conf = &waaad_config;
	
	/* Now set the default values */
	return conf_setdefaults(g_conf);
}
	

/* Parse the configuration file (using the yacc parser) */
int conf_parse ( void )
{
	extern FILE * yyin;
	
	int ret;
	
	if (!g_conf) {
		TRACE_DEBUG (FULL, "Configuration is not initialized...");
		return EINVAL;
	}

	TRACE_DEBUG (INFO, "Parsing configuration file: %s...", g_conf->filepath);
	
	yyin = fopen(g_conf->filepath, "r");
	if (yyin == NULL) {
		ret = errno;
		log_error("Unable to open configuration file %s for reading: %s\n", g_conf->filepath, strerror(ret));
		TRACE_DEBUG (FULL, "Error occurred, message logged -- configuration file.");
		return ret;
	}

	ret = yyparse(g_conf);

	fclose(yyin);

	if (ret != 0) {
		TRACE_DEBUG (INFO, "Unable to parse the configuration file.");
		return EINVAL;
	}
	
	conf_dump();

	return 0;
}	

/* Terminate the configuration module. */
int conf_fini ( void )
{
	/* Nothing to do yet */
	return 0;
}
